<?php

namespace FiloBlu\Flow\Model\Actions\From;

use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;

class Name {

    CONST ENTITY_TYPE_CODE = 'catalog_product';
    protected $_helper_product;
    protected $_product_factory;
    protected $_connection;
    protected $_productMetadata;
    protected $_eavAttributeModel;


    /**
     * @var ProductUrlRewriteGenerator
     */
    protected $productUrlRewriteGenerator;

    /**
     * @var UrlPersistInterface
     */
    protected $urlPersist;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        UrlPersistInterface $urlPersist,
        \Magento\Framework\App\ResourceConnection $_connection,
        \Magento\Framework\App\ProductMetadataInterface $_productMetadata,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute $_eavAttributeModel,
        array $data = []
    ) {
        $this->_helper_product = $helper_product;
        $this->_product_factory = $productFactory;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->urlPersist = $urlPersist;
        $this->_connection = $_connection->getConnection();
        $this->_productMetadata = $_productMetadata;
        $this->_eavAttributeModel = $_eavAttributeModel;
    }

    public function process($product, $value, $storeId) {

        if(empty($value)){
            return true;
        }

        $parent_product_id = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable')->getParentIdsByChild($product->getId());
        if(isset($parent_product_id[0])){

            $fatherProduct = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product')->load($parent_product_id[0]);
            if($fatherProduct->getUrlKey() != $fatherProduct->formatUrlKey($value)){
                $this->setProductName($fatherProduct, $value, $storeId);

            }
        }

        $this->setProductName($product, $value, $storeId);
        return true;
    }

    protected function getEntityField() {

        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise') ? 'row_id' : 'entity_id';

    }

    protected function checkDuplicated($product,$urlKey,$storeId) {

        $result= null;
        $_product_id = $product->getId();
        $entity_field = $this->getEntityField();
        $attribute_id = $this->_eavAttributeModel->getIdByCode('catalog_product', 'url_key');
        $_id_condition = $_product_id ? ' and '.$entity_field.' <> '.$_product_id : '';
        $query = "select * from catalog_product_entity_varchar where attribute_id = ".$attribute_id." and store_id = ".$storeId." and value = '" .$urlKey. "'" .  $_id_condition;
        $result = $this->_connection->fetchRow($query);

        // torno la stringa corretta

        if(is_array($result)) {
            return $urlKey . '-' . $_product_id;
        } else {
            return $urlKey;
        }

    }

    protected function setProductName($product, $name, $storeId){

        $urlKey = $product->formatUrlKey($name);
        $urlKey = $this->checkDuplicated($product,$urlKey,$storeId);

        $pids = array($product->getId());
        $attributesData = array('name' => $name, 'url_key' => $urlKey);

        try{
            \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')
                ->updateAttributes($pids, $attributesData, $storeId);

            $urls = $this->productUrlRewriteGenerator->generate($product);

            $this->urlPersist->deleteByData([
                UrlRewrite::ENTITY_ID => $product->getId(),
                UrlRewrite::ENTITY_TYPE => self::ENTITY_TYPE_CODE,
                UrlRewrite::REDIRECT_TYPE => 0,
                UrlRewrite::STORE_ID => $product->getStoreId()
            ]);

            if ($product->isVisibleInSiteVisibility()) {
                $this->urlPersist->replace($urls);
            }


        } catch (\Exception $e){
            $message = "Problem while processing eavs, <br/> store id : ".$storeId."<br/> pids : ".print_r($pids,true). "<br/> attribute data : ".print_r($attributesData,true);
            $message .= "<br/> \Exception : ".$e->getMessage();
            $message .= "<br/> Backtrace : <br/> ".nl2br($e->getTraceAsString());
            throw new \Exception($e->getMessage());
        }

    }

}

